% many_particle_sim_test.m
%
% Simple script to call various functions to create a 2D image with "N"
% randomly positioned model points, track them, 
% and calculate the localization accuracy using various methods.
% Very simple: all particles have the same brightness and size, and there
% are no checks for overlapping particles.
%
% Raghuveer Parthasarathy
% July 3, 2012

clear all; close all; home
disp(' ');
disp('many_particle_sim_test.m:')
% Create the model 2D image, with many single-particles from modelimage.m
% Parameters:
N = 9;    % Each particle image is calculated in a N x N box. 
          % (Note that the default scale is 100 nm/px)
Np = 75; % Number of particles
SNr = 10.0;  % Signal-to-noise ratio
Nimage = [600 800];  % image size (px)
fs = sprintf('Creating model image, %dx%d with %d particles, SNr = %.1f...', ...
    Nimage(1), Nimage(2), Np, SNr); disp(fs)
[im xc yc]  = modelNparticleimage(SNr, Nimage, N, Np);  % simulated image and "true" positions
disp('  ...done');

%% Track particles
objsize = [3 7];  % filtering size (1st element) -- smaller than N, since I made N rather large 
                  % to capture "all" of the simulated particle.  Making the
                  % filter too large smears nearby particles.  
                  % neighborhood size (2nd element) -- also smaller than N
thresh = 0.999;  % intensity threshold

disp(' ')
disp('Tracking:')
disp(' ')

% Gaussian MLE
fitstr = 'Gaussmle';
disp('Gaussian MLE tracking')
tic
objs_mle = fo4_rp(im, objsize, thresh, fitstr);
e_mle = toc;
fs = sprintf('   ...done.  Elapsed time %.4f seconds.', e_mle); disp(fs)

% Radial-symmetry-based tracking
fitstr = 'radial';
disp('Radial-symmetry tracking')
tic
objs_r = fo4_rp(im, objsize, thresh, fitstr);
e_r = toc;
fs = sprintf('   ...done.  Elapsed time %.4f seconds.', e_r); disp(fs)

% centroid
fitstr = 'centroid';
disp('Centroid finding')
tic
objs_ctr = fo4_rp(im, objsize, thresh, fitstr);
e_ctr = toc;
fs = sprintf('   ...done.  Elapsed time %.4f seconds.', e_ctr); disp(fs)

% which corresponds to which particle?
% find based on a Euclidean distance matrix (similar to distance.m, Roland
% Bunschoten; MATLAB file exchange)
a = [xc'; yc'];
aa=sum(a.*a,1); 
b = objs_r(1:2,:);
bb=sum(b.*b,1); 
d = sqrt(abs(aa( ones(size(bb,2),1), :)' + bb( ones(size(aa,2),1), :) - 2*a'*b));
% d is the matrix of distances; length(true centers) x length(found centers)
Nfound = size(d,2);
if Nfound ~= Np
    disp('*** Warning ***')
    disp('  Number of particles found is not equal to the number of simulated')
    disp('  particles, probably due to overlapping particle images, which this')
    disp('  simple test function doesn''t prevent.  Suggest re-running.')
end
% inelegant:
idmap = zeros(Nfound,1);
for j=1:Nfound
    idmap(j) = find(d(:,j)==min(d(:,j),[],1));
end

% Determine accuracy
disp(' ')
disp('Assessing accuracy:')

% Gaussian MLE
dx_mle = objs_mle(1,:)-xc(idmap)';
dy_mle = objs_mle(2,:)-yc(idmap)';
mx_mle = mean(dx_mle);
my_mle = mean(dy_mle);
std_mle = sqrt(mean(dx_mle.*dx_mle + dy_mle.*dy_mle));
fs = sprintf('Gaussian MLE.'); disp(fs)
fs = sprintf('   Mean Delta_x = %.3f px; Mean Delta_y = %.3f px', mx_mle, my_mle); disp(fs)
fs = sprintf('   Total error (std. of discrepancy) = %.3f px', std_mle); disp(fs)

% Radial sym.
dx_r = objs_r(1,:)-xc(idmap)';
dy_r = objs_r(2,:)-yc(idmap)';
mx_r = mean(dx_r);
my_r = mean(dy_r);
std_r = sqrt(mean(dx_r.*dx_r + dy_r.*dy_r));
fs = sprintf('Radial symmetry.'); disp(fs)
fs = sprintf('   Mean Delta_x = %.3f px; Mean Delta_y = %.3f px', mx_r, my_r); disp(fs)
fs = sprintf('   Total error (std. of discrepancy) = %.3f px', std_r); disp(fs)

% Centroid
dx_ctr = objs_ctr(1,:)-xc(idmap)';
dy_ctr = objs_ctr(2,:)-yc(idmap)';
mx_ctr = mean(dx_ctr);
my_ctr = mean(dy_ctr);
std_ctr = sqrt(mean(dx_ctr.*dx_ctr + dy_ctr.*dy_ctr));
fs = sprintf('Centroid.'); disp(fs)
fs = sprintf('   Mean Delta_x = %.3f px; Mean Delta_y = %.3f px', mx_ctr, my_ctr); disp(fs)
fs = sprintf('   Total error (std. of discrepancy) = %.3f px', std_ctr); disp(fs)

% Output from TrackingGUI
% first run: TrackingGUI_rp(im);
%    Save output (MAT file)
%
% load TrackGUIoutput
% dx_GUI = objs(1,:)-xc(idmap)';
% dy_GUI = objs(2,:)-yc(idmap)';
% mx_GUI = mean(dx_GUI);
% my_GUI = mean(dy_GUI);
% std_GUI = sqrt(mean(dx_GUI.*dx_GUI + dy_GUI.*dy_GUI));
% fs = sprintf('output from GUI.'); disp(fs)
% fs = sprintf('   Mean Delta_x = %.3f px; Mean Delta_y = %.3f px', mx_GUI, my_GUI); disp(fs)
% fs = sprintf('   Total error (std. of discrepancy) = %.3f px', std_GUI); disp(fs)


% Display the simulated image
figure('name', 'simulated image'); imshow(im, []);
