% modelNparticleimage.m
% 
% Uses modelimage.m to create a single image containing several (possibly
% overlapping) fluorescent point particles.
% Each model particle image is calculated in a NxN box with a random center position
% distributed over +/- 0.5 px.  The model image is a point source convolved
% with the point-spread function, pixelated, and with incorporated noise.
% Each of these NxN arrays is placed in the 2D output array.
%
% modelimage.m calls psf2d.m; and uses the poisson-distributed random 
% number generator poissrnd from the MATLAB Statistics toolbox
%
% Inputs
%  SNr  : Signal/noise ratio.  Assumes Poisson noise with mean =
%        sqrt(intensity), so that the simulated CCD image is scaled to have
%        the appropriate peak amplitude . 
%        Default 20.
%  Nimage : the ttal image size (Nimage(1) rows x Nimage(2) columns).  
%      If only one value is input, use this for both dimensions.
%      Default 200
%  N : size of CCD image (NxN pixels).  Force to be an odd integer to ensure
%      centering in the middle of a pixel (default 7)
%  Np : number of particles to simulate (defualt 9)
%  bkg : background (dark) intensity.  (default 10.0)
%  lambda : wavelength of light (nm) (default 530 nm)
%  NA : numerical aperture (default 1.3)
%  dhr : grid size of the "high resolution" image (default 2 nm) -- note
%      that this may determine the simulated tracking precision for high
%      Signal/Noise
% Outputs
%   im : simulated 2D image.  Size Nimage(1)xNimage(2), containing N
%      particles.
%   xc, yc : object centers, x- and y-coordinate (px) 
%      % Note in pixels nm, not nm!
%      [y increases with increasing row number (i.e. "downward")]
%  
% Raghuveer Parthasarathy
% July 3, 2012
% last modified July 13, 2012


function [im xc yc]  = modelNparticleimage(SNr, Nimage, N, Np, bkg, lambda, NA, dhr)


%% defaults for input parameters
if ~exist('SNr', 'var') || isempty(SNr)
    SNr = 20.0; 
end

if ~exist('Nimage', 'var') || isempty(Nimage)
    N = [200 200];
end
if length(Nimage)==1
    Nimage = repmat(Nimage, [1 2]);
end

if ~exist('N', 'var') || isempty(N)
    N = 7;
end
if mod(N,2)==0
    N = N+1;
    fs = sprintf('Forcing N to be odd: %d', N); disp(fs)
end

if ~exist('Np', 'var') || isempty(Np)
    Np = 9;
end
if ~exist('bkg', 'var') || isempty(bkg)
    bkg = 10.0; 
end
if ~exist('lambda', 'var') || isempty(lambda)
    lambda = 530; % nm
end
if ~exist('NA', 'var') || isempty(NA)
    NA = 1.3; 
end

if ~exist('dhr', 'var') || isempty(dhr)
    % resolution of the "high resolution" image, nm/px
    dhr = 1;
end

% Irrelevant "scale" -- needed for input to modelimage.m to be in px.
scale = 100.0; %nm
maxx0nm = 0.5*scale;  % max displacement, *nm*

% Calculate random particle centers, between N+1 and Nimage-(N+1)
xc = (Nimage(2)-2*(N+1))*rand(Np,1) + N + 1;  % avoid edges
yc = (Nimage(1)-2*(N+1))*rand(Np,1) + N + 1;  % avoid edges

% Calculate the psf over a "large" area, if it wasn't input
maxsize = (N-1)*scale + 2.0*maxx0nm;  % maximum needed size for PSF, nm
npsf = round(maxsize/dhr/2.0);
bigpsf = psf2d(npsf, dhr/1000, NA, 1.3, lambda/1000);  % the point-spread function

imsingleparticles  = modelimage(SNr, N, (xc-round(xc))*scale, ...
    (yc-round(yc))*scale, bkg, scale, lambda, NA, dhr, maxx0nm, bigpsf);

im = zeros([Nimage(1) Nimage(2)]);
filled = false([Nimage(1) Nimage(2)]);
% Each particle:
for j=1:Np
    yrange = round(yc(j))-(N-1)/2:round(yc(j))+(N-1)/2;
    xrange = round(xc(j))-(N-1)/2:round(xc(j))+(N-1)/2;
    im(yrange, xrange) =  im(yrange, xrange) + imsingleparticles(:,:,j);
    filled(yrange, xrange) = true;
end
% background: Poisson-distributed noise in places not "filled in" by
% modelimage.m
bkgimage = poissrnd(bkg*ones([Nimage(1) Nimage(2)]));
im(~filled) = bkgimage(~filled);
